library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
entity lrb is
    Port ( clk : in std_logic_vector(2 downto 0);
           clk33 : in std_logic;
           clk_66 : in std_logic;
           DINa : in std_logic_vector(23 downto 0);
           DINb : in std_logic_vector(23 downto 0);
           DINc : in std_logic_vector(23 downto 0);
           SD : in std_logic_vector(35 downto 0);
           SQ : out std_logic_vector(35 downto 0);
           ADR : out std_logic_vector(18 downto 0);
           nWR : out std_logic;
           RCS : out std_logic;
           nRST : in std_logic;
           IDSEL : in std_logic;
           AD : inout std_logic_vector(31 downto 0);
           nCBE : in std_logic_vector(3 downto 0);
           PAR : inout std_logic;
           nFRAME : in std_logic;
           nIRDY : in std_logic;
           nDEVSEL : out std_logic;
           nTRDY : out std_logic;
           nSTOP : out std_logic;
           nPERR : out std_logic;
           nSERR : out std_logic;
           DATA_A : out std_logic;
           DATA_B : out std_logic;
           DATA_C : out std_logic;
           SPARE : out std_logic;
           SPARE_OLD : out std_logic_vector(2 downto 0)
         );
end lrb;

architecture my_arch of lrb is
COMPONENT clklock
	GENERIC (INPUT_FREQUENCY: POSITIVE := 65;
		CLOCKBOOST: POSITIVE);
	PORT (inclk: IN STD_LOGIC;
		outclk: OUT STD_LOGIC);
END COMPONENT;
COMPONENT hamming
	PORT(
		clk : IN std_logic;
		din : IN std_logic_vector(23 downto 0);
		cc_ecc : IN std_logic;
		cc_rxena : IN std_logic;          
		dout : OUT std_logic_vector(19 downto 0)
		);
END COMPONENT;
COMPONENT lvds
	PORT(
		clk : IN std_logic;
		clk133 : IN std_logic;
		hq : IN std_logic_vector(19 downto 0);
		cc_rst : IN std_logic;
		gc_block : IN std_logic;
		gc_idmatch : IN std_logic;
		gc_tstrip : IN std_logic;
		gc_efifo : IN std_logic;
		gc_automon : IN std_logic;
		inh : IN std_logic;
		cs_full : IN std_logic;
		cs_over : IN std_logic;
		ch_number : IN std_logic_vector(1 downto 0);
		lfifo_read : IN std_logic;          
		fqw : OUT std_logic_vector(32 downto 0);
		trailer : OUT std_logic;
		dav_lfifo : OUT std_logic;
		high_lfifo : OUT std_logic;
		update : OUT std_logic;
		WCOUNT : OUT std_logic;
		BADID : OUT std_logic;
		UERR : OUT std_logic;
		CERR : OUT std_logic
		);
END COMPONENT;
COMPONENT ram
	PORT(
		clk : IN std_logic;
		CC_RST : IN std_logic_vector(2 downto 0);
		ch_dav : IN std_logic_vector(2 downto 0);
		ch0_data : IN std_logic_vector(32 downto 0);
		ch1_data : IN std_logic_vector(32 downto 0);
		ch2_data : IN std_logic_vector(32 downto 0);
		GC_IFIFO : IN std_logic;
		GC_RFIFO : IN std_logic;
		GC_OVRL : IN std_logic_vector(1 downto 0);
		pfifo_read : IN std_logic_vector(2 downto 0);
		pfifo_ch : IN std_logic_vector(1 downto 0);          
		ch_read : OUT std_logic_vector(2 downto 0);
		nWR : OUT std_logic;
		RCS : OUT std_logic;
		adr : OUT std_logic_vector(18 downto 0);
		sd : IN std_logic_vector(35 downto 0);
		sq : OUT std_logic_vector(35 downto 0);
		CS_OVER : OUT std_logic_vector(2 downto 0);
		CS_FULL : OUT std_logic_vector(2 downto 0);
		CS_EMPTY : OUT std_logic_vector(2 downto 0);
		pfifo_header : OUT std_logic_vector(2 downto 0);
		pfifo_we : OUT std_logic;
		pfifo_wa : OUT std_logic_vector(7 downto 0);
		pfifo_data : OUT std_logic_vector(32 downto 0)
		);
END COMPONENT;
COMPONENT pci
	PORT(
		clk33 : IN std_logic;
		clk133 : IN std_logic;
		nRST : IN std_logic;
		IDSEL : IN std_logic;
		nCBE : IN std_logic_vector(3 downto 0);
		nFRAME : IN std_logic;
		nIRDY : IN std_logic;
		lfifo_high : IN std_logic_vector(2 downto 0);
		update : IN std_logic_vector(2 downto 0);
		cerr : IN std_logic_vector(2 downto 0);
		uerr : IN std_logic_vector(2 downto 0);
		BADID : IN std_logic_vector(2 downto 0);
		WCOUNT : IN std_logic_vector(2 downto 0);
		fifo_di : IN std_logic_vector(32 downto 0);
		fifo_we : IN std_logic;
		fifo_wa : IN std_logic_vector(7 downto 0);
		trailer : IN std_logic_vector(2 downto 0);
		header : IN std_logic_vector(2 downto 0);
		CS_EMPTY : IN std_logic_vector(2 downto 0);
		CS_OVER : IN std_logic_vector(2 downto 0);
		CS_FULL : IN std_logic_vector(2 downto 0);    
		AD : INOUT std_logic_vector(31 downto 0);
		PAR : INOUT std_logic;
		nTRDY : OUT std_logic;      
		nDEVSEL : OUT std_logic;
		nSTOP : OUT std_logic;
		nPERR : OUT std_logic;
		nSERR : OUT std_logic;
		pci_read : OUT std_logic_vector(2 downto 0);
		pci_ch : OUT std_logic_vector(1 downto 0);
		cc_ecc : OUT std_logic_vector(2 downto 0);
		rst_out : OUT std_logic_vector(2 downto 0);
		gc_block : OUT std_logic;
		gc_tstrip : OUT std_logic;
		gc_idmatch : OUT std_logic;
		gc_efifo : OUT std_logic;
		gc_ififo : OUT std_logic;
		gc_rfifo : OUT std_logic;
		gc_ovrl : OUT std_logic_vector(1 downto 0);
		gc_automon : OUT std_logic;
		cc_rxena : OUT std_logic_vector(2 downto 0);
		data_av : OUT std_logic_vector(2 downto 0)
		);
END COMPONENT;
signal gnd: std_logic;
signal ch0: std_logic_vector(1 downto 0);
signal ch1: std_logic_vector(1 downto 0);
signal ch2: std_logic_vector(1 downto 0);
signal clk133: std_logic;
signal cc_ecc: std_logic_vector(2 downto 0);
signal cc_rxena: std_logic_vector(2 downto 0);
signal hq0: std_logic_vector(19 downto 0);
signal hq1: std_logic_vector(19 downto 0);
signal hq2: std_logic_vector(19 downto 0);
signal cc_rst: std_logic_vector(2 downto 0);
signal gc_block: std_logic;
signal gc_idmatch: std_logic;
signal gc_tstrip: std_logic;
signal gc_ififo: std_logic;
signal gc_rfifo: std_logic;
signal gc_efifo: std_logic;
signal gc_automon: std_logic;
signal cs_full: std_logic_vector(2 downto 0);
signal cs_over: std_logic_vector(2 downto 0);
signal fqw0: std_logic_vector(32 downto 0);
signal fqw1: std_logic_vector(32 downto 0);
signal fqw2: std_logic_vector(32 downto 0);
signal lfifo_read: std_logic_vector(2 downto 0);
signal trailer: std_logic_vector(2 downto 0);
signal header: std_logic_vector(2 downto 0);
signal dav_lfifo: std_logic_vector(2 downto 0);
signal high_lfifo: std_logic_vector(2 downto 0);
signal update: std_logic_vector(2 downto 0);
signal WCOUNT: std_logic_vector(2 downto 0);
signal BADID: std_logic_vector(2 downto 0);
signal UERR: std_logic_vector(2 downto 0);
signal CERR: std_logic_vector(2 downto 0);
signal GC_OVRL: std_logic_vector(1 downto 0);
signal pfifo_ch: std_logic_vector(1 downto 0);
signal pfifo_read: std_logic_vector(2 downto 0);
signal CS_EMPTY: std_logic_vector(2 downto 0);
signal pfifo_we: std_logic;
signal pfifo_wa: std_logic_vector(7 downto 0);
signal pfifo_data: std_logic_vector(32 downto 0);
signal data_av: std_logic_vector(2 downto 0);
begin
gnd <= '0';
ch0 <= "00";
ch1 <= "01";
ch2 <= "10";
i_clk133: clklock generic map(CLOCKBOOST => 2) port map(inclk => clk_66, outclk => clk133);
ham0: hamming PORT MAP(
		clk => clk(0),
		din => DINa,
		cc_ecc => cc_ecc(0),
		dout => hq0,
		cc_rxena => cc_rxena(0) 
	);
ham1: hamming PORT MAP(
		clk => clk(1),
		din => DINb,
		cc_ecc => cc_ecc(1),
		dout => hq1,
		cc_rxena => cc_rxena(1) 
	);
ham2: hamming PORT MAP(
		clk => clk(2),
		din => DINc,
		cc_ecc => cc_ecc(2),
		dout => hq2,
		cc_rxena => cc_rxena(2) 
	);
i_lvds0: lvds PORT MAP(
		clk => clk(0),
		clk133 => clk133,
		hq => hq0,
		cc_rst => cc_rst(0),
		gc_block => gc_block,
		gc_idmatch => gc_idmatch,
		gc_tstrip => gc_tstrip,
		gc_efifo => gc_efifo,
        gc_automon => gc_automon,
		inh => high_lfifo(2),
		cs_full => cs_full(0),
		cs_over => cs_over(0),
		ch_number => ch0,
		fqw => fqw0,
        trailer => trailer(0),
		lfifo_read => lfifo_read(0),
		dav_lfifo => dav_lfifo(0),
		high_lfifo => high_lfifo(0),
		update => update(0),
		WCOUNT => WCOUNT(0),
		BADID => BADID(0),
		UERR => UERR(0),
		CERR => CERR(0)
	);
i_lvds1: lvds PORT MAP(
		clk => clk(1),
		clk133 => clk133,
		hq => hq1,
		cc_rst => cc_rst(1),
		gc_block => gc_block,
		gc_idmatch => gc_idmatch,
		gc_tstrip => gc_tstrip,
		gc_efifo => gc_efifo,
        gc_automon => gc_automon,
		inh => high_lfifo(0),
		cs_full => cs_full(1),
		cs_over => cs_over(1),
		ch_number => ch1,
		fqw => fqw1,
        trailer => trailer(1),
		lfifo_read => lfifo_read(1),
		dav_lfifo => dav_lfifo(1),
		high_lfifo => high_lfifo(1),
		update => update(1),
		WCOUNT => WCOUNT(1),
		BADID => BADID(1),
		UERR => UERR(1),
		CERR => CERR(1)
	);
i_lvds2: lvds PORT MAP(
		clk => clk(2),
		clk133 => clk133,
		hq => hq2,
		cc_rst => cc_rst(2),
		gc_block => gc_block,
		gc_idmatch => gc_idmatch,
		gc_tstrip => gc_tstrip,
		gc_efifo => gc_efifo,
        gc_automon => gc_automon,
		inh => high_lfifo(1),
		cs_full => cs_full(2),
		cs_over => cs_over(2),
		ch_number => ch2,
		fqw => fqw2,
        trailer => trailer(2),
		lfifo_read => lfifo_read(2),
		dav_lfifo => dav_lfifo(2),
		high_lfifo => high_lfifo(2),
		update => update(2),
		WCOUNT => WCOUNT(2),
		BADID => BADID(2),
		UERR => UERR(2),
		CERR => CERR(2)
	);
sram: ram PORT MAP(
		clk => clk133,
		CC_RST => cc_rst,
		ch_dav => dav_lfifo,
		ch0_data => fqw0,
		ch1_data => fqw1,
		ch2_data => fqw2,
		ch_read => lfifo_read,
		nWR => NWR,
		RCS => RCS,
		adr => ADR,
		sd => SD,
		sq => SQ,
		GC_IFIFO => gc_ififo,
		GC_RFIFO => gc_rfifo,
		GC_OVRL => GC_OVRL,
		CS_OVER => cs_over,
		CS_FULL => cs_full,
        pfifo_header => header,
		pfifo_read => pfifo_read,
		pfifo_ch => pfifo_ch,
		CS_EMPTY => CS_EMPTY,
		pfifo_we => pfifo_we,
		pfifo_wa => pfifo_wa,
		pfifo_data => pfifo_data
	);
i_pci: pci PORT MAP(
		clk33 => clk33,
		clk133 => clk133,
		nRST => nRST,
		IDSEL => IDSEL,
		AD => AD,
		nCBE => nCBE,
		PAR => PAR,
		nFRAME => nFRAME,
		nIRDY => nIRDY,
		nDEVSEL => nDEVSEL,
		nTRDY => NTRDY,
		nSTOP => nSTOP,
		nPERR => nPERR,
		nSERR => nSERR,
        gc_automon => gc_automon,
		lfifo_high => high_lfifo,
		update => update,
		cerr => CERR,
		uerr => UERR,
		BADID => BADID,
		WCOUNT => WCOUNT,
		fifo_di => pfifo_data,
		fifo_we => pfifo_we,
		fifo_wa => pfifo_wa,
        header => header,
		trailer => trailer,
		CS_EMPTY => CS_EMPTY,
		CS_OVER => CS_OVER,
		CS_FULL => CS_FULL,
		pci_read => pfifo_read,
		pci_ch => pfifo_ch,
		cc_ecc => CC_ECC,
		rst_out => CC_RST,
		gc_block => gc_block,
		gc_tstrip => gc_tstrip,
		gc_idmatch => gc_idmatch,
		gc_efifo => gc_efifo,
		gc_ififo => gc_ififo,
		gc_rfifo => gc_rfifo,
		gc_ovrl => gc_ovrl,
		cc_rxena => cc_rxena,
		data_av => data_av
	);
DATA_A <= data_av(0);
DATA_B <= data_av(1);
DATA_C <= data_av(2);
SPARE <= '0';
SPARE_OLD <= "000";
end my_arch;
